@@ -16,7 +16,7 @@
 */
 #include <string>
 #include <vector>
-#include <gz/math/Pose3.hh>
+#include <ignition/math/Pose3.hh>
 #include "sdf/Actor.hh"
 #include "sdf/Error.hh"
 #include "Utils.hh"
@@ -24,7 +24,7 @@
 using namespace sdf;
 
 /// \brief Animation private data.
-class sdf::AnimationPrivate
+class sdf::Animation::Implementation
 {
   /// \brief Unique name for animation.
   public: std::string name = "__default__";
@@ -43,17 +43,17 @@ class sdf::AnimationPrivate
 };
 
 /// \brief Waypoint private data.
-class sdf::WaypointPrivate
+class sdf::Waypoint::Implementation
 {
   /// \brief Time to indicate when the pose should be reached.
   public: double time = 0.0;
 
   /// \brief Pose to be reached.
-  public: gz::math::Pose3d pose = gz::math::Pose3d::Zero;
+  public: ignition::math::Pose3d pose = ignition::math::Pose3d::Zero;
 };
 
 /// \brief Trajectory private data.
-class sdf::TrajectoryPrivate
+class sdf::Trajectory::Implementation
 {
     /// \brief Unique id for a trajectory.
     public: uint64_t id = 0;
@@ -69,13 +69,13 @@ class sdf::TrajectoryPrivate
 };
 
 /// \brief Actor private data.
-class sdf::ActorPrivate
+class sdf::Actor::Implementation
 {
   /// \brief Name of the actor.
   public: std::string name = "__default__";
 
   /// \brief Pose of the actor.
-  public: gz::math::Pose3d pose = gz::math::Pose3d::Zero;
+  public: ignition::math::Pose3d pose = ignition::math::Pose3d::Zero;
 
   /// \brief Frame of the actor.
   public: std::string poseRelativeTo = "";
@@ -116,53 +116,10 @@ class sdf::ActorPrivate
 
 /////////////////////////////////////////////////
 Animation::Animation()
-  : dataPtr(new AnimationPrivate)
+  : dataPtr(ignition::utils::MakeImpl<Implementation>())
 {
 }
 
-/////////////////////////////////////////////////
-Animation::~Animation()
-{
-  delete this->dataPtr;
-  this->dataPtr = nullptr;
-}
-
-//////////////////////////////////////////////////
-Animation::Animation(const Animation &_animation)
-  : dataPtr(new AnimationPrivate(*_animation.dataPtr))
-{
-}
-
-/////////////////////////////////////////////////
-Animation::Animation(Animation &&_animation) noexcept
-  : dataPtr(std::exchange(_animation.dataPtr, nullptr))
-{
-}
-
-//////////////////////////////////////////////////
-Animation &Animation::operator=(const Animation &_animation)
-{
-  return *this = Animation(_animation);
-}
-
-//////////////////////////////////////////////////
-Animation &Animation::operator=(Animation &&_animation)
-{
-  std::swap(this->dataPtr, _animation.dataPtr);
-  return *this;
-}
-
-/////////////////////////////////////////////////
-void Animation::CopyFrom(const Animation &_animation)
-{
-  // TODO(anyone) Deprecate function
-  this->dataPtr->name = _animation.dataPtr->name;
-  this->dataPtr->filename = _animation.dataPtr->filename;
-  this->dataPtr->scale = _animation.dataPtr->scale;
-  this->dataPtr->interpolateX = _animation.dataPtr->interpolateX;
-  this->dataPtr->filePath = _animation.dataPtr->filePath;
-}
-
 /////////////////////////////////////////////////
 Errors Animation::Load(ElementPtr _sdf)
 {
@@ -256,50 +213,10 @@ void Animation::SetFilePath(const std::string &_filePath)
 
 /////////////////////////////////////////////////
 Waypoint::Waypoint()
-  : dataPtr(new WaypointPrivate)
+  : dataPtr(ignition::utils::MakeImpl<Implementation>())
 {
 }
 
-/////////////////////////////////////////////////
-Waypoint::~Waypoint()
-{
-  delete this->dataPtr;
-  this->dataPtr = nullptr;
-}
-
-//////////////////////////////////////////////////
-Waypoint::Waypoint(const Waypoint &_waypoint)
-  : dataPtr(new WaypointPrivate(*_waypoint.dataPtr))
-{
-}
-
-/////////////////////////////////////////////////
-Waypoint::Waypoint(Waypoint &&_waypoint) noexcept
-  : dataPtr(std::exchange(_waypoint.dataPtr, nullptr))
-{
-}
-
-//////////////////////////////////////////////////
-Waypoint &Waypoint::operator=(const Waypoint &_waypoint)
-{
-  return *this = Waypoint(_waypoint);
-}
-
-//////////////////////////////////////////////////
-Waypoint &Waypoint::operator=(Waypoint &&_waypoint)
-{
-  std::swap(this->dataPtr, _waypoint.dataPtr);
-  return *this;
-}
-
-/////////////////////////////////////////////////
-void Waypoint::CopyFrom(const Waypoint &_waypoint)
-{
-  // TODO(anyone) deprecate function
-  this->dataPtr->time = _waypoint.dataPtr->time;
-  this->dataPtr->pose = _waypoint.dataPtr->pose;
-}
-
 /////////////////////////////////////////////////
 Errors Waypoint::Load(ElementPtr _sdf)
 {
@@ -313,7 +230,7 @@ Errors Waypoint::Load(ElementPtr _sdf)
   }
   this->dataPtr->time = timeValue.first;
 
-  std::pair posePair = _sdf->Get<gz::math::Pose3d>
+  std::pair posePair = _sdf->Get<ignition::math::Pose3d>
                         ("pose", this->dataPtr->pose);
   if (!posePair.second)
   {
@@ -338,65 +255,23 @@ void Waypoint::SetTime(double _time)
 }
 
 /////////////////////////////////////////////////
-gz::math::Pose3d Waypoint::Pose() const
+ignition::math::Pose3d Waypoint::Pose() const
 {
   return this->dataPtr->pose;
 }
 
 /////////////////////////////////////////////////
-void Waypoint::SetPose(const gz::math::Pose3d &_pose)
+void Waypoint::SetPose(const ignition::math::Pose3d &_pose)
 {
   this->dataPtr->pose = _pose;
 }
 
 /////////////////////////////////////////////////
 Trajectory::Trajectory()
-  : dataPtr(new TrajectoryPrivate)
+  : dataPtr(ignition::utils::MakeImpl<Implementation>())
 {
 }
 
-/////////////////////////////////////////////////
-Trajectory::~Trajectory()
-{
-  delete this->dataPtr;
-  this->dataPtr = nullptr;
-}
-
-//////////////////////////////////////////////////
-Trajectory::Trajectory(const Trajectory &_trajectory)
-  : dataPtr(new TrajectoryPrivate(*_trajectory.dataPtr))
-{
-}
-
-/////////////////////////////////////////////////
-Trajectory::Trajectory(Trajectory &&_trajectory) noexcept
-  : dataPtr(std::exchange(_trajectory.dataPtr, nullptr))
-{
-}
-
-//////////////////////////////////////////////////
-Trajectory &Trajectory::operator=(const Trajectory &_trajectory)
-{
-  return *this = Trajectory(_trajectory);
-}
-
-//////////////////////////////////////////////////
-Trajectory &Trajectory::operator=(Trajectory &&_trajectory)
-{
-  std::swap(this->dataPtr, _trajectory.dataPtr);
-  return *this;
-}
-
-/////////////////////////////////////////////////
-void Trajectory::CopyFrom(const Trajectory &_trajectory)
-{
-  // TODO(anyone) deprecate function
-  this->dataPtr->id = _trajectory.dataPtr->id;
-  this->dataPtr->type = _trajectory.dataPtr->type;
-  this->dataPtr->tension = _trajectory.dataPtr->tension;
-  this->dataPtr->waypoints = _trajectory.dataPtr->waypoints;
-}
-
 /////////////////////////////////////////////////
 Errors Trajectory::Load(ElementPtr _sdf)
 {
@@ -488,59 +363,10 @@ void Trajectory::AddWaypoint(const Waypoint &_waypoint)
 
 /////////////////////////////////////////////////
 Actor::Actor()
-  : dataPtr(new ActorPrivate)
+  : dataPtr(ignition::utils::MakeImpl<Implementation>())
 {
 }
 
-/////////////////////////////////////////////////
-Actor::~Actor()
-{
-  delete this->dataPtr;
-  this->dataPtr = nullptr;
-}
-
-//////////////////////////////////////////////////
-Actor::Actor(const Actor &_actor)
-  : dataPtr(new ActorPrivate(*_actor.dataPtr))
-{
-}
-
-/////////////////////////////////////////////////
-Actor::Actor(Actor &&_actor) noexcept
-    : dataPtr(std::exchange(_actor.dataPtr, nullptr))
-{
-}
-
-//////////////////////////////////////////////////
-Actor &Actor::operator=(const Actor &_actor)
-{
-  return *this = Actor(_actor);
-}
-
-//////////////////////////////////////////////////
-Actor &Actor::operator=(Actor &&_actor)
-{
-  std::swap(this->dataPtr, _actor.dataPtr);
-  return *this;
-}
-
-//////////////////////////////////////////////////
-void Actor::CopyFrom(const Actor &_actor)
-{
-  // TODO(anyone) deprecate function
-  this->dataPtr->name             = _actor.dataPtr->name;
-  this->dataPtr->pose             = _actor.dataPtr->pose;
-  this->dataPtr->poseRelativeTo   = _actor.dataPtr->poseRelativeTo;
-  this->dataPtr->skinFilename     = _actor.dataPtr->skinFilename;
-  this->dataPtr->skinScale        = _actor.dataPtr->skinScale;
-  this->dataPtr->animations       = _actor.dataPtr->animations;
-  this->dataPtr->scriptLoop       = _actor.dataPtr->scriptLoop;
-  this->dataPtr->scriptDelayStart = _actor.dataPtr->scriptDelayStart;
-  this->dataPtr->scriptAutoStart  = _actor.dataPtr->scriptAutoStart;
-  this->dataPtr->trajectories     = _actor.dataPtr->trajectories;
-  this->dataPtr->filePath         = _actor.dataPtr->filePath;
-}
-
 /////////////////////////////////////////////////
 Errors Actor::Load(ElementPtr _sdf)
 {
@@ -631,7 +457,7 @@ Errors Actor::Load(ElementPtr _sdf)
 }
 
 /////////////////////////////////////////////////
-std::string &Actor::Name() const
+const std::string &Actor::Name() const
 {
   return this->dataPtr->name;
 }
@@ -643,23 +469,11 @@ void Actor::SetName(const std::string &_name)
 }
 
 /////////////////////////////////////////////////
-const gz::math::Pose3d &Actor::Pose() const
-{
-  return this->RawPose();
-}
-
-/////////////////////////////////////////////////
-const gz::math::Pose3d &Actor::RawPose() const
+const ignition::math::Pose3d &Actor::RawPose() const
 {
   return this->dataPtr->pose;
 }
 
-/////////////////////////////////////////////////
-const std::string &Actor::PoseFrame() const
-{
-  return this->PoseRelativeTo();
-}
-
 /////////////////////////////////////////////////
 const std::string &Actor::PoseRelativeTo() const
 {
@@ -667,23 +481,11 @@ const std::string &Actor::PoseRelativeTo() const
 }
 
 /////////////////////////////////////////////////
-void Actor::SetPose(const gz::math::Pose3d &_pose)
-{
-  this->SetRawPose(_pose);
-}
-
-/////////////////////////////////////////////////
-void Actor::SetRawPose(const gz::math::Pose3d &_pose)
+void Actor::SetRawPose(const ignition::math::Pose3d &_pose)
 {
   this->dataPtr->pose = _pose;
 }
 
-/////////////////////////////////////////////////
-void Actor::SetPoseFrame(const std::string &_frame)
-{
-  this->SetPoseRelativeTo(_frame);
-}
-
 /////////////////////////////////////////////////
 void Actor::SetPoseRelativeTo(const std::string &_frame)
 {
