@@ -46,19 +46,16 @@ static std::array<std::string, 19> kPixelFormatNames =
 };
 
 // Private data class
-class sdf::CameraPrivate
+class sdf::Camera::Implementation
 {
   /// \brief The SDF element pointer used during load.
   public: sdf::ElementPtr sdf;
 
-  /// \brief Camera info topic.
-  public: std::string cameraInfoTopic = "";
-
   /// \brief Name of the camera.
   public: std::string name = "";
 
   /// \brief Horizontal fied of view.
-  public: gz::math::Angle hfov{1.047};
+  public: ignition::math::Angle hfov{1.047};
 
   /// \brief Image width.
   public: uint32_t imageWidth{320};
@@ -69,9 +66,6 @@ class sdf::CameraPrivate
   /// \brief Image format.
   public: PixelFormatType pixelFormat{PixelFormatType::RGB_INT8};
 
-  /// \brief Anti-aliasing value.
-  public: uint32_t antiAliasingValue{4};
-
   /// \brief Near clip distance.
   public: double nearClip{0.1};
 
@@ -118,10 +112,10 @@ class sdf::CameraPrivate
   public: double distortionP2{0.0};
 
   /// \brief The distortion center or principal point
-  public: gz::math::Vector2d distortionCenter{0.5, 0.5};
+  public: ignition::math::Vector2d distortionCenter{0.5, 0.5};
 
   /// \brief Pose of the link
-  public: gz::math::Pose3d pose = gz::math::Pose3d::Zero;
+  public: ignition::math::Pose3d pose = ignition::math::Pose3d::Zero;
 
   /// \brief Frame of the pose.
   public: std::string poseRelativeTo = "";
@@ -148,7 +142,7 @@ class sdf::CameraPrivate
   public: std::string lensFun{"tan"};
 
   /// \brief Lens cutoff angle.
-  public: gz::math::Angle lensCutoffAngle{IGN_PI_2};
+  public: ignition::math::Angle lensCutoffAngle{IGN_PI_2};
 
   /// \brief lens environment texture size.
   public: int lensEnvTextureSize{256};
@@ -165,72 +159,19 @@ class sdf::CameraPrivate
   /// \brief lens instrinsics cy.
   public: double lensIntrinsicsCy{120.0};
 
-  /// \brief lens projection fx
-  public: double lensProjectionFx{277.0};
-
-  /// \brief lens projection fy
-  public: double lensProjectionFy{277.0};
-
-  /// \brief lens projection cx
-  public: double lensProjectionCx{160.0};
-
-  /// \brief lens projection cy
-  public: double lensProjectionCy{120.0};
-
-  /// \brief lens projection tx
-  public: double lensProjectionTx{0.0};
-
-  /// \brief lens projection ty
-  public: double lensProjectionTy{0.0};
-
   /// \brief lens instrinsics s.
   public: double lensIntrinsicsS{1.0};
 
-  /// \brief True if this camera has custom projection values
-  public: bool hasProjection = false;
-
   /// \brief Visibility mask of a camera. Defaults to 0xFFFFFFFF
   public: uint32_t visibilityMask{4294967295u};
 };
 
 /////////////////////////////////////////////////
 Camera::Camera()
-  : dataPtr(new CameraPrivate)
+  : dataPtr(ignition::utils::MakeImpl<Implementation>())
 {
 }
 
-/////////////////////////////////////////////////
-Camera::~Camera()
-{
-  delete this->dataPtr;
-  this->dataPtr = nullptr;
-}
-
-/////////////////////////////////////////////////
-Camera::Camera(const Camera &_camera)
-  : dataPtr(new CameraPrivate(*_camera.dataPtr))
-{
-}
-
-/////////////////////////////////////////////////
-Camera::Camera(Camera &&_camera) noexcept
-  : dataPtr(std::exchange(_camera.dataPtr, nullptr))
-{
-}
-
-//////////////////////////////////////////////////
-Camera &Camera::operator=(const Camera &_camera)
-{
-  return *this = Camera(_camera);
-}
-
-//////////////////////////////////////////////////
-Camera &Camera::operator=(Camera &&_camera) noexcept
-{
-  std::swap(this->dataPtr, _camera.dataPtr);
-  return *this;
-}
-
 /////////////////////////////////////////////////
 Errors Camera::Load(ElementPtr _sdf)
 {
@@ -259,12 +200,7 @@ Errors Camera::Load(ElementPtr _sdf)
   // Read the camera's name
   loadName(_sdf, this->dataPtr->name);
 
-  this->dataPtr->cameraInfoTopic = _sdf->Get<std::string>("camera_info_topic",
-      this->dataPtr->cameraInfoTopic).first;
-  if (this->dataPtr->cameraInfoTopic == "__default__")
-    this->dataPtr->cameraInfoTopic = "";
-
-  this->dataPtr->hfov = _sdf->Get<gz::math::Angle>("horizontal_fov",
+  this->dataPtr->hfov = _sdf->Get<ignition::math::Angle>("horizontal_fov",
       this->dataPtr->hfov).first;
 
   // Read the distortion
@@ -283,7 +219,7 @@ Errors Camera::Load(ElementPtr _sdf)
     this->dataPtr->distortionP2 = elem->Get<double>("p2",
       this->dataPtr->distortionP2).first;
 
-    this->dataPtr->distortionCenter = elem->Get<gz::math::Vector2d>(
+    this->dataPtr->distortionCenter = elem->Get<ignition::math::Vector2d>(
         "center", this->dataPtr->distortionCenter).first;
   }
 
@@ -302,10 +238,6 @@ Errors Camera::Load(ElementPtr _sdf)
       errors.push_back({ErrorCode::ELEMENT_INVALID,
         "Camera sensor <image><format> has invalid value of " + format});
     }
-
-    this->dataPtr->antiAliasingValue =
-        elem->Get<uint32_t>("anti_aliasing",
-            this->dataPtr->antiAliasingValue).first;
   }
   else
   {
@@ -380,7 +312,7 @@ Errors Camera::Load(ElementPtr _sdf)
         this->dataPtr->lensType).first;
     this->dataPtr->lensScaleToHfov = elem->Get<bool>("scale_to_hfov",
         this->dataPtr->lensScaleToHfov).first;
-    this->dataPtr->lensCutoffAngle = elem->Get<gz::math::Angle>(
+    this->dataPtr->lensCutoffAngle = elem->Get<ignition::math::Angle>(
         "cutoff_angle", this->dataPtr->lensCutoffAngle).first;
     this->dataPtr->lensEnvTextureSize = elem->Get<int>("env_texture_size",
         this->dataPtr->lensEnvTextureSize).first;
@@ -414,23 +346,6 @@ Errors Camera::Load(ElementPtr _sdf)
       this->dataPtr->lensIntrinsicsS = intrinsics->Get<double>("s",
           this->dataPtr->lensIntrinsicsS).first;
     }
-
-    if (elem->HasElement("projection")) {
-      sdf::ElementPtr projection = elem->GetElement("projection");
-      this->dataPtr->lensProjectionFx = projection->Get<double>("p_fx",
-          this->dataPtr->lensProjectionFx).first;
-      this->dataPtr->lensProjectionFy = projection->Get<double>("p_fy",
-          this->dataPtr->lensProjectionFy).first;
-      this->dataPtr->lensProjectionCx = projection->Get<double>("p_cx",
-          this->dataPtr->lensProjectionCx).first;
-      this->dataPtr->lensProjectionCy = projection->Get<double>("p_cy",
-          this->dataPtr->lensProjectionCy).first;
-      this->dataPtr->lensProjectionTx = projection->Get<double>("tx",
-          this->dataPtr->lensProjectionTx).first;
-      this->dataPtr->lensProjectionTy = projection->Get<double>("ty",
-          this->dataPtr->lensProjectionTy).first;
-      this->dataPtr->hasProjection = true;
-    }
   }
 
   if (_sdf->HasElement("visibility_mask"))
@@ -442,18 +357,6 @@ Errors Camera::Load(ElementPtr _sdf)
   return errors;
 }
 
-/////////////////////////////////////////////////
-std::string Camera::CameraInfoTopic() const
-{
-  return this->dataPtr->cameraInfoTopic;
-}
-
-/////////////////////////////////////////////////
-void Camera::SetCameraInfoTopic(const std::string &_cameraInfoTopic)
-{
-  this->dataPtr->cameraInfoTopic = _cameraInfoTopic;
-}
-
 /////////////////////////////////////////////////
 sdf::ElementPtr Camera::Element() const
 {
@@ -473,13 +376,13 @@ void Camera::SetName(const std::string &_name)
 }
 
 /////////////////////////////////////////////////
-gz::math::Angle Camera::HorizontalFov() const
+ignition::math::Angle Camera::HorizontalFov() const
 {
   return this->dataPtr->hfov;
 }
 
 /////////////////////////////////////////////////
-void Camera::SetHorizontalFov(const gz::math::Angle &_hfov)
+void Camera::SetHorizontalFov(const ignition::math::Angle &_hfov)
 {
   this->dataPtr->hfov = _hfov;
 }
@@ -532,18 +435,6 @@ void Camera::SetPixelFormatStr(const std::string &_fmt)
   this->dataPtr->pixelFormat = ConvertPixelFormat(_fmt);
 }
 
-//////////////////////////////////////////////////
-uint32_t Camera::AntiAliasingValue() const
-{
-  return this->dataPtr->antiAliasingValue;
-}
-
-//////////////////////////////////////////////////
-void Camera::SetAntiAliasingValue(uint32_t _antiAliasingValue)
-{
-  this->dataPtr->antiAliasingValue = _antiAliasingValue;
-}
-
 //////////////////////////////////////////////////
 double Camera::DepthNearClip() const
 {
@@ -662,8 +553,8 @@ bool Camera::operator==(const Camera &_cam) const
     this->ImageWidth() == _cam.ImageWidth() &&
     this->ImageHeight() == _cam.ImageHeight() &&
     this->PixelFormat() == _cam.PixelFormat() &&
-    gz::math::equal(this->NearClip(), _cam.NearClip()) &&
-    gz::math::equal(this->FarClip(), _cam.FarClip()) &&
+    ignition::math::equal(this->NearClip(), _cam.NearClip()) &&
+    ignition::math::equal(this->FarClip(), _cam.FarClip()) &&
     this->SaveFrames() == _cam.SaveFrames() &&
     this->SaveFramesPath() == _cam.SaveFramesPath() &&
     this->ImageNoise() == _cam.ImageNoise() &&
@@ -749,36 +640,23 @@ void Camera::SetDistortionP2(double _p2)
 }
 
 //////////////////////////////////////////////////
-const gz::math::Vector2d &Camera::DistortionCenter() const
+const ignition::math::Vector2d &Camera::DistortionCenter() const
 {
   return this->dataPtr->distortionCenter;
 }
 
 //////////////////////////////////////////////////
-void Camera::SetDistortionCenter(
-                const gz::math::Vector2d &_center) const
+void Camera::SetDistortionCenter(const ignition::math::Vector2d &_center)
 {
   this->dataPtr->distortionCenter = _center;
 }
 
 /////////////////////////////////////////////////
-const gz::math::Pose3d &Camera::Pose() const
-{
-  return this->RawPose();
-}
-
-/////////////////////////////////////////////////
-const gz::math::Pose3d &Camera::RawPose() const
+const ignition::math::Pose3d &Camera::RawPose() const
 {
   return this->dataPtr->pose;
 }
 
-/////////////////////////////////////////////////
-const std::string &Camera::PoseFrame() const
-{
-  return this->PoseRelativeTo();
-}
-
 /////////////////////////////////////////////////
 const std::string &Camera::PoseRelativeTo() const
 {
@@ -786,23 +664,11 @@ const std::string &Camera::PoseRelativeTo() const
 }
 
 /////////////////////////////////////////////////
-void Camera::SetPose(const gz::math::Pose3d &_pose)
-{
-  this->SetRawPose(_pose);
-}
-
-/////////////////////////////////////////////////
-void Camera::SetRawPose(const gz::math::Pose3d &_pose)
+void Camera::SetRawPose(const ignition::math::Pose3d &_pose)
 {
   this->dataPtr->pose = _pose;
 }
 
-/////////////////////////////////////////////////
-void Camera::SetPoseFrame(const std::string &_frame)
-{
-  this->SetPoseRelativeTo(_frame);
-}
-
 /////////////////////////////////////////////////
 void Camera::SetPoseRelativeTo(const std::string &_frame)
 {
@@ -894,13 +760,13 @@ void Camera::SetLensFunction(const std::string &_fun)
 }
 
 /////////////////////////////////////////////////
-gz::math::Angle Camera::LensCutoffAngle() const
+ignition::math::Angle Camera::LensCutoffAngle() const
 {
   return this->dataPtr->lensCutoffAngle;
 }
 
 /////////////////////////////////////////////////
-void Camera::SetLensCutoffAngle(const gz::math::Angle &_angle)
+void Camera::SetLensCutoffAngle(const ignition::math::Angle &_angle)
 {
   this->dataPtr->lensCutoffAngle = _angle;
 }
@@ -965,84 +831,6 @@ void Camera::SetLensIntrinsicsCy(double _cy)
   this->dataPtr->lensIntrinsicsCy = _cy;
 }
 
-/////////////////////////////////////////////////
-double Camera::LensProjectionFx() const
-{
-  return this->dataPtr->lensProjectionFx;
-}
-
-/////////////////////////////////////////////////
-void Camera::SetLensProjectionFx(double _fx_p)
-{
-  this->dataPtr->lensProjectionFx = _fx_p;
-  this->dataPtr->hasProjection = true;
-}
-
-/////////////////////////////////////////////////
-double Camera::LensProjectionFy() const
-{
-  return this->dataPtr->lensProjectionFy;
-}
-
-/////////////////////////////////////////////////
-void Camera::SetLensProjectionFy(double _fy_p)
-{
-  this->dataPtr->lensProjectionFy = _fy_p;
-  this->dataPtr->hasProjection = true;
-}
-
-/////////////////////////////////////////////////
-double Camera::LensProjectionCx() const
-{
-  return this->dataPtr->lensProjectionCx;
-}
-
-/////////////////////////////////////////////////
-void Camera::SetLensProjectionCx(double _cx_p)
-{
-  this->dataPtr->lensProjectionCx = _cx_p;
-  this->dataPtr->hasProjection = true;
-}
-
-/////////////////////////////////////////////////
-double Camera::LensProjectionCy() const
-{
-  return this->dataPtr->lensProjectionCy;
-}
-
-/////////////////////////////////////////////////
-void Camera::SetLensProjectionCy(double _cy_p)
-{
-  this->dataPtr->lensProjectionCy = _cy_p;
-  this->dataPtr->hasProjection = true;
-}
-
-/////////////////////////////////////////////////
-double Camera::LensProjectionTx() const
-{
-  return this->dataPtr->lensProjectionTx;
-}
-
-/////////////////////////////////////////////////
-void Camera::SetLensProjectionTx(double _tx)
-{
-  this->dataPtr->lensProjectionTx = _tx;
-  this->dataPtr->hasProjection = true;
-}
-
-/////////////////////////////////////////////////
-double Camera::LensProjectionTy() const
-{
-  return this->dataPtr->lensProjectionTy;
-}
-
-/////////////////////////////////////////////////
-void Camera::SetLensProjectionTy(double _ty)
-{
-  this->dataPtr->lensProjectionTy = _ty;
-  this->dataPtr->hasProjection = true;
-}
-
 /////////////////////////////////////////////////
 double Camera::LensIntrinsicsSkew() const
 {
@@ -1108,9 +896,3 @@ void Camera::SetVisibilityMask(uint32_t _mask)
 {
   this->dataPtr->visibilityMask = _mask;
 }
-
-/////////////////////////////////////////////////
-bool Camera::HasLensProjection() const
-{
-  return this->dataPtr->hasProjection;
-}
