@@ -284,11 +284,28 @@ TEST(Element, Include)
 {
   sdf::Element elem;
 
+  SDF_SUPPRESS_DEPRECATED_BEGIN
   ASSERT_EQ(elem.GetInclude(), "");
 
   elem.SetInclude("foo.txt");
 
   ASSERT_EQ(elem.GetInclude(), "foo.txt");
+  SDF_SUPPRESS_DEPRECATED_END
+
+  auto includeElemToStore = std::make_shared<sdf::Element>();
+  includeElemToStore->SetName("include");
+  auto uriDesc = std::make_shared<sdf::Element>();
+  uriDesc->SetName("uri");
+  uriDesc->AddValue("string", "", true);
+  includeElemToStore->AddElementDescription(uriDesc);
+
+  includeElemToStore->GetElement("uri")->Set("foo.txt");
+  elem.SetIncludeElement(includeElemToStore);
+
+  auto includeElem = elem.GetIncludeElement();
+  ASSERT_NE(nullptr, includeElem);
+  ASSERT_TRUE(includeElem->HasElement("uri"));
+  EXPECT_EQ("foo.txt", includeElem->GetElement("uri")->Get<std::string>());
 }
 
 /////////////////////////////////////////////////
@@ -400,15 +417,26 @@ TEST(Element, Clone)
   parent->AddValue("string", "foo", false, "foo description");
 
   parent->SetFilePath("/path/to/file.sdf");
+  parent->SetLineNumber(12);
+  parent->SetXmlPath("/sdf/world[@name=\"default\"]");
   parent->SetOriginalVersion("1.5");
 
+  auto includeElemToStore = std::make_shared<sdf::Element>();
+  includeElemToStore->SetName("include");
+  parent->SetIncludeElement(includeElemToStore);
+
   sdf::ElementPtr newelem = parent->Clone();
 
   EXPECT_EQ("/path/to/file.sdf", newelem->FilePath());
+  ASSERT_TRUE(newelem->LineNumber().has_value());
+  EXPECT_EQ(12, newelem->LineNumber().value());
+  EXPECT_EQ("/sdf/world[@name=\"default\"]", newelem->XmlPath());
   EXPECT_EQ("1.5", newelem->OriginalVersion());
   ASSERT_NE(newelem->GetFirstElement(), nullptr);
   ASSERT_EQ(newelem->GetElementDescriptionCount(), 1UL);
   ASSERT_EQ(newelem->GetAttributeCount(), 1UL);
+  ASSERT_NE(newelem->GetIncludeElement(), nullptr);
+  EXPECT_EQ("include", newelem->GetIncludeElement()->GetName());
   ASSERT_TRUE(newelem->GetExplicitlySetInFile());
 }
 
@@ -419,11 +447,16 @@ TEST(Element, ClearElements)
   sdf::ElementPtr child = std::make_shared<sdf::Element>();
 
   parent->SetFilePath("/path/to/file.sdf");
+  parent->SetLineNumber(12);
+  parent->SetXmlPath("/sdf/world[@name=\"default\"]");
   parent->SetOriginalVersion("1.5");
   child->SetParent(parent);
   parent->InsertElement(child);
 
   EXPECT_EQ("/path/to/file.sdf", parent->FilePath());
+  ASSERT_TRUE(parent->LineNumber().has_value());
+  EXPECT_EQ(12, parent->LineNumber().value());
+  EXPECT_EQ("/sdf/world[@name=\"default\"]", parent->XmlPath());
   EXPECT_EQ("1.5", parent->OriginalVersion());
   ASSERT_NE(parent->GetFirstElement(), nullptr);
   EXPECT_EQ("/path/to/file.sdf", parent->GetFirstElement()->FilePath());
@@ -433,6 +466,8 @@ TEST(Element, ClearElements)
 
   ASSERT_EQ(parent->GetFirstElement(), nullptr);
   EXPECT_EQ("/path/to/file.sdf", parent->FilePath());
+  ASSERT_TRUE(parent->LineNumber().has_value());
+  EXPECT_EQ(12, parent->LineNumber().value());
   EXPECT_EQ("1.5", parent->OriginalVersion());
 }
 
@@ -443,11 +478,16 @@ TEST(Element, Clear)
   sdf::ElementPtr child = std::make_shared<sdf::Element>();
 
   parent->SetFilePath("/path/to/file.sdf");
+  parent->SetLineNumber(12);
+  parent->SetXmlPath("/sdf/world[@name=\"default\"]");
   parent->SetOriginalVersion("1.5");
   child->SetParent(parent);
   parent->InsertElement(child);
 
   EXPECT_EQ("/path/to/file.sdf", parent->FilePath());
+  ASSERT_TRUE(parent->LineNumber().has_value());
+  EXPECT_EQ(12, parent->LineNumber().value());
+  EXPECT_EQ("/sdf/world[@name=\"default\"]", parent->XmlPath());
   EXPECT_EQ("1.5", parent->OriginalVersion());
   ASSERT_NE(parent->GetFirstElement(), nullptr);
   EXPECT_EQ("/path/to/file.sdf", parent->GetFirstElement()->FilePath());
@@ -457,6 +497,8 @@ TEST(Element, Clear)
 
   ASSERT_EQ(parent->GetFirstElement(), nullptr);
   EXPECT_TRUE(parent->FilePath().empty());
+  EXPECT_FALSE(parent->LineNumber().has_value());
+  EXPECT_TRUE(parent->XmlPath().empty());
   EXPECT_TRUE(parent->OriginalVersion().empty());
 }
 
@@ -564,11 +606,13 @@ TEST(Element, ToStringInclude)
 {
   sdf::Element elem;
 
+  SDF_SUPPRESS_DEPRECATED_BEGIN
   ASSERT_EQ(elem.GetInclude(), "");
 
   elem.SetInclude("foo.txt");
 
   ASSERT_EQ(elem.GetInclude(), "foo.txt");
+  SDF_SUPPRESS_DEPRECATED_END
 
   std::string stringval = elem.ToString("myprefix");
   ASSERT_EQ(stringval, "myprefix<include filename='foo.txt'/>\n");
@@ -727,9 +771,6 @@ TEST(Element, Set)
   elem.AddValue("string", "val", false, "val description");
 
   ASSERT_TRUE(elem.Set<std::string>("hello"));
-
-  elem.AddValue("int", "0", true, "value");
-  ASSERT_FALSE(elem.Set<std::string>(""));
 }
 
 /////////////////////////////////////////////////
@@ -740,14 +781,23 @@ TEST(Element, Copy)
 
   src->SetName("test");
   src->SetFilePath("/path/to/file.sdf");
+  src->SetLineNumber(12);
+  src->SetXmlPath("/sdf/world[@name=\"default\"]");
   src->SetOriginalVersion("1.5");
   src->AddValue("string", "val", false, "val description");
   src->AddAttribute("test", "string", "foo", false, "foo description");
   src->InsertElement(std::make_shared<sdf::Element>());
 
+  auto includeElemToStore = std::make_shared<sdf::Element>();
+  includeElemToStore->SetName("include");
+  src->SetIncludeElement(includeElemToStore);
+
   dest->Copy(src);
 
   EXPECT_EQ("/path/to/file.sdf", dest->FilePath());
+  ASSERT_TRUE(dest->LineNumber().has_value());
+  EXPECT_EQ(12, dest->LineNumber().value());
+  EXPECT_EQ("/sdf/world[@name=\"default\"]", dest->XmlPath());
   EXPECT_EQ("1.5", dest->OriginalVersion());
 
   sdf::ParamPtr param = dest->GetValue();
@@ -767,6 +817,8 @@ TEST(Element, Copy)
   ASSERT_EQ(param->GetDescription(), "foo description");
 
   ASSERT_NE(dest->GetFirstElement(), nullptr);
+  ASSERT_NE(dest->GetIncludeElement(), nullptr);
+  EXPECT_EQ("include", dest->GetIncludeElement()->GetName());
 }
 
 /////////////////////////////////////////////////
