@@ -44,7 +44,6 @@ inline namespace SDF_VERSION_NAMESPACE {
 // Helpful functions when debugging in gdb
 void printGraph(const ScopedGraph<PoseRelativeToGraph> &_graph)
 {
-  std::stringstream ss;
   std::cout << _graph.Graph() << std::endl;
 }
 void printGraph(const ScopedGraph<FrameAttachedToGraph> &_graph)
@@ -258,7 +257,7 @@ std::pair<const Link *, std::string>
 /// be modified.
 static Errors resolveModelPoseWithPlacementFrame(
     const ignition::math::Pose3d &_rawPose,
-    const std::string _placementFrame,
+    const std::string &_placementFrame,
     const sdf::ScopedGraph<PoseRelativeToGraph> &_graph,
     ignition::math::Pose3d &_resolvedPose)
 {
@@ -314,7 +313,6 @@ Errors buildFrameAttachedToGraph(
 
   const std::string scopeContextName = "__model__";
 
-  auto rootId = ignition::math::graph::kNullId;
   if (_isRoot)
   {
     // The __root__ vertex identifies the scope that contains a root level
@@ -327,7 +325,6 @@ Errors buildFrameAttachedToGraph(
     // more complexity to the validateFrameAttachedToGraph code.
     _out = _out.AddScopeVertex(
         "", "__root__", scopeContextName, sdf::FrameType::STATIC_MODEL);
-    rootId = _out.ScopeVertexId();
   }
 
   const auto modelId = _out.AddVertex(_model->Name(), frameType).Id();
