@@ -16,10 +16,12 @@
  */
 
 #include <algorithm>
+#include <cctype>
 #include <cstdint>
 #include <locale>
 #include <sstream>
 #include <string>
+#include <vector>
 
 #include <locale.h>
 #include <math.h>
@@ -71,6 +73,42 @@ Param::Param(const std::string &_key, const std::string &_typeName,
   this->dataPtr->defaultValue = this->dataPtr->value;
 }
 
+//////////////////////////////////////////////////
+Param::Param(const std::string &_key, const std::string &_typeName,
+             const std::string &_default, bool _required,
+             const std::string &_minValue, const std::string &_maxValue,
+             const std::string &_description)
+    : Param(_key, _typeName, _default, _required, _description)
+{
+  auto valCopy = this->dataPtr->value;
+  if (!_minValue.empty())
+  {
+    SDF_ASSERT(
+        this->ValueFromString(_minValue),
+        std::string("Invalid [min] parameter in SDFormat description of [") +
+            _key + "]");
+    this->dataPtr->minValue = this->dataPtr->value;
+  }
+
+  if (!_maxValue.empty())
+  {
+    SDF_ASSERT(
+        this->ValueFromString(_maxValue),
+        std::string("Invalid [max] parameter in SDFormat description of [") +
+            _key + "]");
+    this->dataPtr->maxValue = this->dataPtr->value;
+  }
+
+  this->dataPtr->value = valCopy;
+}
+
+Param::Param(const Param &_param)
+    : dataPtr(std::make_unique<ParamPrivate>(*_param.dataPtr))
+{
+  // We don't want to copy the updateFunc
+  this->dataPtr->updateFunc = nullptr;
+}
+
 //////////////////////////////////////////////////
 Param::~Param()
 {
@@ -79,9 +117,11 @@ Param::~Param()
 /////////////////////////////////////////////////
 Param &Param::operator=(const Param &_param)
 {
-  this->dataPtr->value = _param.dataPtr->value;
-  this->dataPtr->defaultValue  = _param.dataPtr->defaultValue;
-  this->dataPtr->set  = _param.dataPtr->set;
+  auto updateFuncCopy = this->dataPtr->updateFunc;
+  *this = Param(_param);
+
+  // Restore the update func
+  this->dataPtr->updateFunc = updateFuncCopy;
   return *this;
 }
 
@@ -169,55 +209,55 @@ bool Param::GetAny(std::any &_anyVal) const
     }
     _anyVal = ret;
   }
-  else if (this->IsType<gz::math::Color>())
+  else if (this->IsType<ignition::math::Color>())
   {
-    gz::math::Color ret;
-    if (!this->Get<gz::math::Color>(ret))
+    ignition::math::Color ret;
+    if (!this->Get<ignition::math::Color>(ret))
     {
       return false;
     }
     _anyVal = ret;
   }
-  else if (this->IsType<gz::math::Vector3d>())
+  else if (this->IsType<ignition::math::Vector3d>())
   {
-    gz::math::Vector3d ret;
-    if (!this->Get<gz::math::Vector3d>(ret))
+    ignition::math::Vector3d ret;
+    if (!this->Get<ignition::math::Vector3d>(ret))
     {
       return false;
     }
     _anyVal = ret;
   }
-  else if (this->IsType<gz::math::Vector2i>())
+  else if (this->IsType<ignition::math::Vector2i>())
   {
-    gz::math::Vector2i ret;
-    if (!this->Get<gz::math::Vector2i>(ret))
+    ignition::math::Vector2i ret;
+    if (!this->Get<ignition::math::Vector2i>(ret))
     {
       return false;
     }
     _anyVal = ret;
   }
-  else if (this->IsType<gz::math::Vector2d>())
+  else if (this->IsType<ignition::math::Vector2d>())
   {
-    gz::math::Vector2d ret;
-    if (!this->Get<gz::math::Vector2d>(ret))
+    ignition::math::Vector2d ret;
+    if (!this->Get<ignition::math::Vector2d>(ret))
     {
       return false;
     }
     _anyVal = ret;
   }
-  else if (this->IsType<gz::math::Pose3d>())
+  else if (this->IsType<ignition::math::Pose3d>())
   {
-    gz::math::Pose3d ret;
-    if (!this->Get<gz::math::Pose3d>(ret))
+    ignition::math::Pose3d ret;
+    if (!this->Get<ignition::math::Pose3d>(ret))
     {
       return false;
     }
     _anyVal = ret;
   }
-  else if (this->IsType<gz::math::Quaterniond>())
+  else if (this->IsType<ignition::math::Quaterniond>())
   {
-    gz::math::Quaterniond ret;
-    if (!this->Get<gz::math::Quaterniond>(ret))
+    ignition::math::Quaterniond ret;
+    if (!this->Get<ignition::math::Quaterniond>(ret))
     {
       return false;
     }
@@ -271,6 +311,122 @@ std::string Param::GetDefaultAsString() const
   return ss.str();
 }
 
+//////////////////////////////////////////////////
+std::optional<std::string> Param::GetMinValueAsString() const
+{
+  if (this->dataPtr->minValue.has_value())
+  {
+    StringStreamClassicLocale ss;
+
+    ss << ParamStreamer{ *this->dataPtr->minValue };
+    return ss.str();
+  }
+  return std::nullopt;
+}
+
+//////////////////////////////////////////////////
+std::optional<std::string> Param::GetMaxValueAsString() const
+{
+  if (this->dataPtr->maxValue.has_value())
+  {
+    StringStreamClassicLocale ss;
+
+    ss << ParamStreamer{ *this->dataPtr->maxValue };
+    return ss.str();
+  }
+  return std::nullopt;
+}
+
+//////////////////////////////////////////////////
+/// \brief Helper function for Param::ValueFromString
+/// \param[in] _input Input string.
+/// \param[in] _key Key of the parameter, used for error message.
+/// \param[out] _value This will be set with the parsed value.
+/// \return True if parsing succeeded.
+template <typename T>
+bool ParseUsingStringStream(const std::string &_input, const std::string &_key,
+                            ParamPrivate::ParamVariant &_value)
+{
+  StringStreamClassicLocale ss(_input);
+  T _val;
+  ss >> _val;
+  if (ss.fail())
+  {
+    sdferr << "Unknown error. Unable to set value [" << _input << " ] for key["
+           << _key << "]\n";
+    return false;
+  }
+  _value = _val;
+  return true;
+}
+
+//////////////////////////////////////////////////
+/// \brief Helper function for Param::ValueFromString for parsing colors
+/// This checks the color components specified in _input are rgb or rgba
+/// (expects 3 or 4 values) and each value is between [0,1]. When only 3 values
+/// are present (rgb), then alpha is set to 1.
+/// \param[in] _input Input string.
+/// \param[in] _key Key of the parameter, used for error message.
+/// \param[out] _value This will be set with the parsed value.
+/// \return True if parsing colors succeeded.
+bool ParseColorUsingStringStream(const std::string &_input,
+    const std::string &_key, ParamPrivate::ParamVariant &_value)
+{
+  StringStreamClassicLocale ss(_input);
+  std::string token;
+  std::vector<float> colors;
+  float c;  // r,g,b,a values
+  bool isValidColor = true;
+  while (ss >> token)
+  {
+    try
+    {
+      c = std::stof(token);
+      colors.push_back(c);
+    }
+    // Catch invalid argument exception from std::stof
+    catch(std::invalid_argument &)
+    {
+      sdferr << "Invalid argument. Unable to set value ["<< token
+             << "] for key [" << _key << "].\n";
+      isValidColor = false;
+      break;
+    }
+    // Catch out of range exception from std::stof
+    catch(std::out_of_range &)
+    {
+      sdferr << "Out of range. Unable to set value [" << token
+             << "] for key [" << _key << "].\n";
+      isValidColor = false;
+      break;
+    }
+
+    if (c < 0.0f || c > 1.0f)
+    {
+      isValidColor = false;
+      break;
+    }
+  }
+
+  size_t colorSize = colors.size();
+  if (isValidColor && colorSize == 3u)
+    colors.push_back(1.0f);
+  else if (colorSize != 4u)
+    isValidColor = false;
+
+  if (isValidColor)
+  {
+    _value = ignition::math::Color(colors[0], colors[1], colors[2], colors[3]);
+  }
+  else
+  {
+    sdferr << "The value <" << _key <<
+        ">" << _input << "</" << _key << "> is invalid.\n";
+  }
+
+  return isValidColor;
+}
+
 //////////////////////////////////////////////////
 bool Param::ValueFromString(const std::string &_value)
 {
@@ -288,9 +444,9 @@ bool ParamPrivate::ValueFromStringImpl(const std::string &_typeName,
   // comma for decimal position instead of a dot, making the conversion
   // to fail. See bug #60 for more information. Force to use always C
   setlocale(LC_NUMERIC, "C");
-
-  std::string tmp(_valueStr);
-  std::string lowerTmp = lowercase(_valueStr);
+  std::string trimmed = sdf::trim(_valueStr);
+  std::string tmp(trimmed);
+  std::string lowerTmp = lowercase(trimmed);
 
   // "true" and "false" doesn't work properly
   if (lowerTmp == "true")
@@ -345,11 +501,8 @@ bool ParamPrivate::ValueFromStringImpl(const std::string &_typeName,
     }
     else if (_typeName == "uint64_t")
     {
-      StringStreamClassicLocale ss(tmp);
-      std::uint64_t u64tmp;
-
-      ss >> u64tmp;
-      _valueToSet = u64tmp;
+      return ParseUsingStringStream<std::uint64_t>(tmp, this->key,
+                                                   _valueToSet);
     }
     else if (_typeName == "unsigned int")
     {
@@ -367,75 +520,53 @@ bool ParamPrivate::ValueFromStringImpl(const std::string &_typeName,
     else if (_typeName == "sdf::Time" ||
              _typeName == "time")
     {
-      StringStreamClassicLocale ss(tmp);
-      sdf::Time timetmp;
-
-      ss >> timetmp;
-      _valueToSet = timetmp;
+      return ParseUsingStringStream<sdf::Time>(tmp, this->key,
+                                               _valueToSet);
     }
-    else if (_typeName == "gz::math::Angle" ||
+    else if (_typeName == "ignition::math::Angle" ||
              _typeName == "angle")
     {
-      StringStreamClassicLocale ss(tmp);
-      gz::math::Angle angletmp;
-
-      ss >> angletmp;
-      _valueToSet = angletmp;
+      return ParseUsingStringStream<ignition::math::Angle>(
+          tmp, this->key, _valueToSet);
     }
-    else if (_typeName == "gz::math::Color" ||
+    else if (_typeName == "ignition::math::Color" ||
              _typeName == "color")
     {
-      StringStreamClassicLocale ss(tmp);
-      gz::math::Color colortmp;
-
-      ss >> colortmp;
-      _valueToSet = colortmp;
+      return ParseColorUsingStringStream(tmp, this->key, _valueToSet);
     }
-    else if (_typeName == "gz::math::Vector2i" ||
+    else if (_typeName == "ignition::math::Vector2i" ||
              _typeName == "vector2i")
     {
-      StringStreamClassicLocale ss(tmp);
-      gz::math::Vector2i vectmp;
-
-      ss >> vectmp;
-      _valueToSet = vectmp;
+      return ParseUsingStringStream<ignition::math::Vector2i>(
+          tmp, this->key, _valueToSet);
     }
-    else if (_typeName == "gz::math::Vector2d" ||
+    else if (_typeName == "ignition::math::Vector2d" ||
              _typeName == "vector2d")
     {
-      StringStreamClassicLocale ss(tmp);
-      gz::math::Vector2d vectmp;
-
-      ss >> vectmp;
-      _valueToSet = vectmp;
+      return ParseUsingStringStream<ignition::math::Vector2d>(
+          tmp, this->key, _valueToSet);
     }
-    else if (_typeName == "gz::math::Vector3d" ||
+    else if (_typeName == "ignition::math::Vector3d" ||
              _typeName == "vector3")
     {
-      StringStreamClassicLocale ss(tmp);
-      gz::math::Vector3d vectmp;
-
-      ss >> vectmp;
-      _valueToSet = vectmp;
+      return ParseUsingStringStream<ignition::math::Vector3d>(
+          tmp, this->key, _valueToSet);
     }
-    else if (_typeName == "gz::math::Pose3d" ||
+    else if (_typeName == "ignition::math::Pose3d" ||
              _typeName == "pose" ||
              _typeName == "Pose")
     {
-      StringStreamClassicLocale ss(tmp);
-      gz::math::Pose3d posetmp;
-
-      ss >> posetmp;
-      _valueToSet = posetmp;
+      if (!tmp.empty())
+      {
+        return ParseUsingStringStream<ignition::math::Pose3d>(
+            tmp, this->key, _valueToSet);
+      }
     }
-    else if (_typeName == "gz::math::Quaterniond" ||
+    else if (_typeName == "ignition::math::Quaterniond" ||
              _typeName == "quaternion")
     {
-      StringStreamClassicLocale ss(tmp);
-      gz::math::Quaterniond quattmp;
-
-      ss >> quattmp;
-      _valueToSet = quattmp;
+      return ParseUsingStringStream<ignition::math::Quaterniond>(
+          tmp, this->key, _valueToSet);
     }
     else
     {
@@ -480,11 +611,19 @@ bool Param::SetFromString(const std::string &_value)
     return true;
   }
 
+  auto oldValue = this->dataPtr->value;
   if (!this->ValueFromString(str))
   {
     return false;
   }
 
+  // Check if the value is permitted
+  if (!this->ValidateValue())
+  {
+    this->dataPtr->value = oldValue;
+    return false;
+  }
+
   this->dataPtr->set = true;
   return this->dataPtr->set;
 }
@@ -499,11 +638,7 @@ void Param::Reset()
 //////////////////////////////////////////////////
 ParamPtr Param::Clone() const
 {
-  ParamPtr clone(new Param(this->dataPtr->key, this->dataPtr->typeName,
-                            this->GetAsString(), this->dataPtr->required,
-                            this->dataPtr->description));
-  clone->dataPtr->set = this->dataPtr->set;
-  return clone;
+  return std::make_shared<Param>(*this);
 }
 
 //////////////////////////////////////////////////
@@ -541,3 +676,41 @@ bool Param::GetSet() const
 {
   return this->dataPtr->set;
 }
+
+/////////////////////////////////////////////////
+bool Param::ValidateValue() const
+{
+  return std::visit(
+      [this](const auto &_val) -> bool
+      {
+        using T = std::decay_t<decltype(_val)>;
+        // cppcheck-suppress syntaxError
+        // cppcheck-suppress unmatchedSuppression
+        if constexpr (std::is_scalar_v<T>)
+        {
+          if (this->dataPtr->minValue.has_value())
+          {
+            if (_val < std::get<T>(*this->dataPtr->minValue))
+            {
+              sdferr << "The value [" << _val
+                     << "] is less than the minimum allowed value of ["
+                     << *this->GetMinValueAsString() << "] for key ["
+                     << this->GetKey() << "]\n";
+              return false;
+            }
+          }
+          if (this->dataPtr->maxValue.has_value())
+          {
+            if (_val > std::get<T>(*this->dataPtr->maxValue))
+            {
+              sdferr << "The value [" << _val
+                     << "] is greater than the maximum allowed value of ["
+                     << *this->GetMaxValueAsString() << "] for key ["
+                     << this->GetKey() << "]\n";
+              return false;
+            }
+          }
+        }
+        return true;
+      }, this->dataPtr->value);
+}
