@@ -508,7 +508,7 @@ bool readFileInternal(const std::string &_filename, const bool _convert,
   {
     URDF2SDF u2g;
     auto doc = makeSdfDoc();
-    u2g.InitModelFile(filename, &doc);
+    u2g.InitModelFile(filename, _config, &doc);
     if (sdf::readDoc(&doc, _sdf, "urdf file", _convert, _config, _errors))
     {
       sdfdbg << "parse from urdf file [" << _filename << "].\n";
@@ -585,7 +585,7 @@ bool readStringInternal(const std::string &_xmlString, const bool _convert,
   {
     URDF2SDF u2g;
     auto doc = makeSdfDoc();
-    u2g.InitModelString(_xmlString, &doc);
+    u2g.InitModelString(_xmlString, _config, &doc);
 
     if (sdf::readDoc(&doc, _sdf, std::string(kUrdfStringSource), _convert,
                     _config, _errors))
@@ -661,6 +661,7 @@ bool readDoc(tinyxml2::XMLDocument *_xmlDoc, SDFPtr _sdf,
   tinyxml2::XMLElement *sdfNode = _xmlDoc->FirstChildElement("sdf");
   if (!sdfNode)
   {
+    sdfdbg << "No <sdf> element in file[" << _source << "]\n";
     return false;
   }
 
@@ -675,7 +676,7 @@ bool readDoc(tinyxml2::XMLDocument *_xmlDoc, SDFPtr _sdf,
     _sdf->SetFilePath(_source);
   }
 
-  if (sdfNode && sdfNode->Attribute("version"))
+  if (sdfNode->Attribute("version"))
   {
     if (_sdf->OriginalVersion().empty())
     {
@@ -737,15 +738,8 @@ bool readDoc(tinyxml2::XMLDocument *_xmlDoc, SDFPtr _sdf,
   }
   else
   {
-    if (!sdfNode)
-    {
-      sdfdbg << "No <sdf> element in file[" << _source << "]\n";
-    }
-    else if (!sdfNode->Attribute("version"))
-    {
-      sdfdbg << "SDF <sdf> element has no version in file["
-             << _source << "]\n";
-    }
+    sdfdbg << "SDF <sdf> element has no version in file["
+           << _source << "]\n";
     return false;
   }
 
@@ -767,6 +761,7 @@ bool readDoc(tinyxml2::XMLDocument *_xmlDoc, ElementPtr _sdf,
   tinyxml2::XMLElement *sdfNode = _xmlDoc->FirstChildElement("sdf");
   if (!sdfNode)
   {
+    sdfdbg << "SDF has no <sdf> element\n";
     return false;
   }
 
@@ -775,7 +770,7 @@ bool readDoc(tinyxml2::XMLDocument *_xmlDoc, ElementPtr _sdf,
     _sdf->SetFilePath(_source);
   }
 
-  if (sdfNode && sdfNode->Attribute("version"))
+  if (sdfNode->Attribute("version"))
   {
     if (_sdf->OriginalVersion().empty())
     {
@@ -838,14 +833,7 @@ bool readDoc(tinyxml2::XMLDocument *_xmlDoc, ElementPtr _sdf,
   }
   else
   {
-    if (!sdfNode)
-    {
-      sdfdbg << "SDF has no <sdf> element\n";
-    }
-    else if (!sdfNode->Attribute("version"))
-    {
-      sdfdbg << "<sdf> element has no version\n";
-    }
+    sdfdbg << "<sdf> element has no version\n";
     return false;
   }
 
@@ -1149,7 +1137,7 @@ static bool readAttributes(tinyxml2::XMLElement *_xml, ElementPtr _sdf,
 /// \param[out] _errors Captures errors found during parsing.
 /// \return True if the file name is successfully resolved, false on error.
 static bool resolveFileNameFromUri(tinyxml2::XMLElement *_includeXml,
-    const sdf::ParserConfig &_config, const std::string _includeXmlPath,
+    const sdf::ParserConfig &_config, const std::string &_includeXmlPath,
     const std::string &_errorSourcePath, std::string &_fileName,
     Errors &_errors)
 {
@@ -2032,7 +2020,8 @@ bool recursiveSiblingUniqueNames(sdf::ElementPtr _elem)
   if (!shouldValidateElement(_elem))
     return true;
 
-  bool result = _elem->HasUniqueChildNames();
+  bool result =
+      _elem->HasUniqueChildNames("", Element::NameUniquenessExceptions());
   if (!result)
   {
     std::cerr << "Error: Non-unique names detected in "
@@ -2244,114 +2233,100 @@ bool checkPoseRelativeToGraph(const sdf::Root *_root)
 //////////////////////////////////////////////////
 bool checkJointParentChildLinkNames(const sdf::Root *_root)
 {
-  bool result = true;
+  Errors errors;
+  checkJointParentChildNames(_root, errors);
+  if (!errors.empty())
+  {
+    std::cerr << "Error when attempting to resolve child link name:"
+              << std::endl
+              << errors;
+    return false;
+  }
+  return true;
+}
 
+//////////////////////////////////////////////////
+void checkJointParentChildNames(const sdf::Root *_root, Errors &_errors)
+{
   auto checkModelJointParentChildNames = [](
-      const sdf::Model *_model) -> bool
+      const sdf::Model *_model, Errors &errors) -> void
   {
-    bool modelResult = true;
     for (uint64_t j = 0; j < _model->JointCount(); ++j)
     {
       auto joint = _model->JointByIndex(j);
 
       const std::string &parentName = joint->ParentLinkName();
-      if (parentName != "world" && !_model->LinkNameExists(parentName) &&
-          !_model->JointNameExists(parentName) &&
-          !_model->FrameNameExists(parentName))
+      const std::string parentLocalName = sdf::SplitName(parentName).second;
+
+      if (parentName != "world" && parentLocalName != "__model__" &&
+          !_model->NameExistsInFrameAttachedToGraph(parentName))
       {
-        std::cerr << "Error: parent frame with name[" << parentName
-                  << "] specified by joint with name[" << joint->Name()
-                  << "] not found in model with name[" << _model->Name()
-                  << "]."
-                  << std::endl;
-        modelResult = false;
+        errors.push_back({ErrorCode::JOINT_PARENT_LINK_INVALID,
+          "parent frame with name[" + parentName +
+          "] specified by joint with name[" + joint->Name() +
+          "] not found in model with name[" + _model->Name() + "]."});
       }
 
       const std::string &childName = joint->ChildLinkName();
+      const std::string childLocalName = sdf::SplitName(childName).second;
       if (childName == "world")
       {
-        std::cerr << "Error: invalid child name[world"
-                  << "] specified by joint with name[" << joint->Name()
-                  << "] in model with name[" << _model->Name()
-                  << "]."
-                  << std::endl;
-        modelResult = false;
+        errors.push_back({ErrorCode::JOINT_CHILD_LINK_INVALID,
+          "invalid child name[world] specified by joint with name[" +
+          joint->Name() + "] in model with name[" + _model->Name() + "]."});
       }
 
-      if (!_model->LinkNameExists(childName) &&
-          !_model->JointNameExists(childName) &&
-          !_model->FrameNameExists(childName) &&
-          !_model->ModelNameExists(childName))
+      if (childLocalName != "__model__" &&
+          !_model->NameExistsInFrameAttachedToGraph(childName))
       {
-        std::cerr << "Error: child frame with name[" << childName
-                  << "] specified by joint with name[" << joint->Name()
-                  << "] not found in model with name[" << _model->Name()
-                  << "]."
-                  << std::endl;
-        modelResult = false;
+        errors.push_back({ErrorCode::JOINT_CHILD_LINK_INVALID,
+          "child frame with name[" + childName +
+          "] specified by joint with name[" + joint->Name() +
+          "] not found in model with name[" + _model->Name() + "]."});
       }
 
       if (childName == joint->Name())
       {
-        std::cerr << "Error: joint with name[" << joint->Name()
-                  << "] in model with name[" << _model->Name()
-                  << "] must not specify its own name as the child frame."
-                  << std::endl;
-        modelResult = false;
+        errors.push_back({ErrorCode::JOINT_CHILD_LINK_INVALID,
+          "joint with name[" + joint->Name() +
+          "] in model with name[" + _model->Name() +
+          "] must not specify its own name as the child frame."});
       }
 
       if (parentName == joint->Name())
       {
-        std::cerr << "Error: joint with name[" << joint->Name()
-                  << "] in model with name[" << _model->Name()
-                  << "] must not specify its own name as the parent frame."
-                  << std::endl;
-        modelResult = false;
+        errors.push_back({ErrorCode::JOINT_PARENT_LINK_INVALID,
+          "joint with name[" + joint->Name() +
+          "] in model with name[" + _model->Name() +
+          "] must not specify its own name as the parent frame."});
       }
 
       // Check that parent and child frames resolve to different links
       std::string resolvedChildName;
       std::string resolvedParentName;
-      auto errors = joint->ResolveChildLink(resolvedChildName);
-      if (!errors.empty())
-      {
-        std::cerr << "Error when attempting to resolve child link name:"
-                  << std::endl;
-        for (auto error : errors)
-        {
-          std::cerr << error.Message() << std::endl;
-        }
-        modelResult = false;
-      }
-      errors = joint->ResolveParentLink(resolvedParentName);
-      if (!errors.empty())
-      {
-        std::cerr << "Error when attempting to resolve parent link name:"
-                  << std::endl;
-        for (auto error : errors)
-        {
-          std::cerr << error.Message() << std::endl;
-        }
-        modelResult = false;
-      }
+
+      auto resolveErrors = joint->ResolveChildLink(resolvedChildName);
+      errors.insert(errors.end(), resolveErrors.begin(), resolveErrors.end());
+
+      resolveErrors = joint->ResolveParentLink(resolvedParentName);
+      errors.insert(errors.end(), resolveErrors.begin(), resolveErrors.end());
+
       if (resolvedChildName == resolvedParentName)
       {
-        std::cerr << "Error: joint with name[" << joint->Name()
-                  << "] in model with name[" << _model->Name()
-                  << "] specified parent frame [" << parentName
-                  << "] and child frame [" << childName
-                  << "] that both resolve to [" << resolvedChildName
-                  << "], but they should resolve to different values."
-                  << std::endl;
-        modelResult = false;
+        errors.push_back({ErrorCode::JOINT_PARENT_SAME_AS_CHILD,
+          "joint with name[" + joint->Name() +
+          "] in model with name[" + _model->Name() +
+          "] specified parent frame [" + parentName +
+          "] and child frame [" + childName +
+          "] that both resolve to [" + resolvedChildName +
+          "], but they should resolve to different values."});
       }
     }
-    return modelResult;
   };
 
   if (_root->Model())
   {
-    result = checkModelJointParentChildNames(_root->Model()) && result;
+    checkModelJointParentChildNames(_root->Model(), _errors);
   }
 
   for (uint64_t w = 0; w < _root->WorldCount(); ++w)
@@ -2360,11 +2335,9 @@ bool checkJointParentChildLinkNames(const sdf::Root *_root)
     for (uint64_t m = 0; m < world->ModelCount(); ++m)
     {
       auto model = world->ModelByIndex(m);
-      result = checkModelJointParentChildNames(model) && result;
+      checkModelJointParentChildNames(model, _errors);
     }
   }
-
-  return result;
 }
 
 //////////////////////////////////////////////////
