@@ -13,52 +13,51 @@
  * See the License for the specific language governing permissions and
  * limitations under the License.
  *
-*/
+ */
+
+#include <string>
 
 #include <gtest/gtest.h>
-#include <boost/filesystem.hpp>
-#include <map>
+
 #include "sdf/sdf.hh"
 
 #include "test_config.h"
 
-const std::string SDF_TEST_FILE = std::string(PROJECT_SOURCE_PATH)
-  + "/test/integration/force_torque_sensor.urdf";
-
 /////////////////////////////////////////////////
 TEST(SDFParser, ForceTorqueSensorTest)
 {
-  char *pathCStr = getenv("SDF_PATH");
-  boost::filesystem::path path = PROJECT_SOURCE_PATH;
-  path = path / "sdf" / SDF_VERSION;
-  setenv("SDF_PATH", path.string().c_str(), 1);
+  const std::string sdfTestFile =
+      sdf::testing::TestFile("integration", "force_torque_sensor.urdf");
 
   sdf::SDFPtr robot(new sdf::SDF());
   sdf::init(robot);
-  ASSERT_TRUE(sdf::readFile(SDF_TEST_FILE, robot));
-  if (pathCStr)
-  {
-    setenv("SDF_PATH", pathCStr, 1);
-  }
-  else
-  {
-    unsetenv("SDF_PATH");
-  }
+  ASSERT_TRUE(sdf::readFile(sdfTestFile, robot));
 
   sdf::ElementPtr model = robot->Root()->GetElement("model");
+  unsigned int jointBitMask = 0;
   for (sdf::ElementPtr joint = model->GetElement("joint"); joint;
        joint = joint->GetNextElement("joint"))
   {
     std::string jointName = joint->Get<std::string>("name");
     if (jointName == "joint_1")
     {
+      jointBitMask |= 0x1;
+
       // Sensor tag was specified
       EXPECT_TRUE(joint->HasElement("sensor"));
     }
     else if (jointName == "joint_2")
     {
+      jointBitMask |= 0x2;
+
       // No sensor tag was specified
       EXPECT_FALSE(joint->HasElement("sensor"));
     }
+    else
+    {
+      FAIL() << "Unexpected joint name[" << jointName << "]";
+    }
   }
+
+  EXPECT_EQ(jointBitMask, 0x3u);
 }
