@@ -55,11 +55,11 @@ TEST(IncludesTest, Includes)
 
   ASSERT_NE(nullptr, root.Element());
   EXPECT_EQ(worldFile, root.Element()->FilePath());
-  EXPECT_EQ("1.6", root.Element()->OriginalVersion());
+  EXPECT_EQ("1.8", root.Element()->OriginalVersion());
 
   const sdf::World *world = root.WorldByIndex(0);
   ASSERT_NE(nullptr, world);
-  EXPECT_EQ("1.6", world->Element()->OriginalVersion());
+  EXPECT_EQ("1.8", world->Element()->OriginalVersion());
 
   // Actors
   EXPECT_EQ(2u, world->ActorCount());
@@ -77,7 +77,7 @@ TEST(IncludesTest, Includes)
   EXPECT_EQ(actorFile, actor->FilePath());
 
   EXPECT_EQ("actor", actor->Name());
-  EXPECT_EQ(gz::math::Pose3d(0, 0, 0, 0, 0, 0), actor->RawPose());
+  EXPECT_EQ(ignition::math::Pose3d(0, 0, 0, 0, 0, 0), actor->RawPose());
   EXPECT_EQ("", actor->PoseRelativeTo());
   EXPECT_EQ("meshes/skin.dae", actor->SkinFilename());
   EXPECT_DOUBLE_EQ(1.0, actor->SkinScale());
@@ -112,7 +112,7 @@ TEST(IncludesTest, Includes)
 
   const auto *actor1 = world->ActorByIndex(1);
   EXPECT_EQ("override_actor_name", actor1->Name());
-  EXPECT_EQ(gz::math::Pose3d(7, 8, 9, 0, 0, 0), actor1->RawPose());
+  EXPECT_EQ(ignition::math::Pose3d(7, 8, 9, 0, 0, 0), actor1->RawPose());
   EXPECT_EQ("", actor1->PoseRelativeTo());
   ASSERT_NE(nullptr, actor1->Element());
   EXPECT_TRUE(actor1->Element()->HasElement("plugin"));
@@ -127,7 +127,7 @@ TEST(IncludesTest, Includes)
   EXPECT_EQ("point_light", pointLight->Name());
   EXPECT_EQ(sdf::LightType::POINT, pointLight->Type());
   EXPECT_FALSE(pointLight->CastShadows());
-  EXPECT_EQ(gz::math::Pose3d(1, 2, 10, 0, 0, 0), pointLight->RawPose());
+  EXPECT_EQ(ignition::math::Pose3d(1, 2, 10, 0, 0, 0), pointLight->RawPose());
   EXPECT_EQ("world", pointLight->PoseRelativeTo());
   EXPECT_DOUBLE_EQ(123.5, pointLight->AttenuationRange());
   EXPECT_DOUBLE_EQ(1.0, pointLight->LinearAttenuationFactor());
@@ -140,11 +140,11 @@ TEST(IncludesTest, Includes)
   const auto *pointLight1 = world->LightByIndex(1);
   ASSERT_NE(nullptr, pointLight1);
   EXPECT_EQ("override_light_name", pointLight1->Name());
-  EXPECT_EQ(gz::math::Pose3d(4, 5, 6, 0, 0, 0), pointLight1->RawPose());
+  EXPECT_EQ(ignition::math::Pose3d(4, 5, 6, 0, 0, 0), pointLight1->RawPose());
   EXPECT_EQ("", pointLight1->PoseRelativeTo());
 
   // Models
-  EXPECT_EQ(2u, world->ModelCount());
+  EXPECT_EQ(3u, world->ModelCount());
   EXPECT_FALSE(world->ModelNameExists(""));
 
   // Model without overrides
@@ -175,7 +175,7 @@ TEST(IncludesTest, Includes)
   ASSERT_NE(nullptr, meshColGeom);
   EXPECT_EQ("meshes/mesh.dae", meshColGeom->Uri());
   EXPECT_EQ(modelFile, meshColGeom->FilePath());
-  EXPECT_TRUE(gz::math::Vector3d(0.1, 0.2, 0.3) ==
+  EXPECT_TRUE(ignition::math::Vector3d(0.1, 0.2, 0.3) ==
       meshColGeom->Scale());
   EXPECT_EQ("my_submesh", meshColGeom->Submesh());
   EXPECT_TRUE(meshColGeom->CenterSubmesh());
@@ -187,7 +187,7 @@ TEST(IncludesTest, Includes)
   const auto *meshVisGeom = meshVis->Geom()->MeshShape();
   EXPECT_EQ(modelFile, meshVisGeom->FilePath());
   EXPECT_EQ("meshes/mesh.dae", meshVisGeom->Uri());
-  EXPECT_TRUE(gz::math::Vector3d(1.2, 2.3, 3.4) ==
+  EXPECT_TRUE(ignition::math::Vector3d(1.2, 2.3, 3.4) ==
       meshVisGeom->Scale());
   EXPECT_EQ("another_submesh", meshVisGeom->Submesh());
   EXPECT_FALSE(meshVisGeom->CenterSubmesh());
@@ -200,10 +200,23 @@ TEST(IncludesTest, Includes)
   ASSERT_NE(nullptr, model1);
   EXPECT_EQ("override_model_name", model1->Name());
   EXPECT_TRUE(model1->Static());
-  EXPECT_EQ(gz::math::Pose3d(1, 2, 3, 0, 0, 0), model1->RawPose());
+  EXPECT_EQ(ignition::math::Pose3d(1, 2, 3, 0, 0, 0), model1->RawPose());
   EXPECT_EQ("", model1->PoseRelativeTo());
   ASSERT_NE(nullptr, model1->Element());
   EXPECT_TRUE(model1->Element()->HasElement("plugin"));
+
+  const sdf::Model *model2 = world->ModelByIndex(2);
+  ASSERT_NE(nullptr, model2);
+  EXPECT_EQ("test_model_with_file", model2->Name());
+  EXPECT_FALSE(model2->Static());
+  EXPECT_EQ(1u, model2->LinkCount());
+  ASSERT_NE(nullptr, model2->LinkByIndex(0));
+  ASSERT_NE(nullptr, model2->LinkByName("link"));
+  EXPECT_EQ(model2->LinkByName("link")->Name(), model2->LinkByIndex(0)->Name());
+  EXPECT_EQ(nullptr, model2->LinkByIndex(1));
+  EXPECT_TRUE(model2->LinkNameExists("link"));
+  EXPECT_FALSE(model2->LinkNameExists("coconut"));
+  EXPECT_EQ("1.6", model2->Element()->OriginalVersion());
 }
 
 //////////////////////////////////////////////////
@@ -346,5 +359,32 @@ TEST(IncludesTest, IncludeModelMissingConfig)
     std::cout << e.Message() << std::endl;
   EXPECT_TRUE(errors.empty());
 
-  EXPECT_EQ(0u, root.ModelCount());
+  EXPECT_EQ(nullptr, root.Model());
 }
+
+//////////////////////////////////////////////////
+/// Check that sdformat natively parses URDF files when there are no custom
+/// parsers
+TEST(IncludesTest, IncludeUrdf)
+{
+  sdf::setFindCallback(findFileCb);
+
+  std::ostringstream stream;
+  stream
+    << "<sdf version='" << SDF_VERSION << "'>"
+    << "<include>"
+    << "  <uri>test_include_urdf</uri>"
+    << "</include>"
+    << "</sdf>";
+
+  sdf::Root root;
+  sdf::Errors errors = root.LoadSdfString(stream.str());
+  ASSERT_TRUE(errors.empty()) << errors;
+
+  auto model = root.Model();
+  ASSERT_NE(nullptr, model);
+  EXPECT_EQ("test_include_urdf", model->Name());
+  EXPECT_EQ(2u, model->LinkCount());
+  EXPECT_EQ(1u, model->JointCount());
+}
+
