@@ -218,7 +218,7 @@ TEST(DOMJoint, LoadInvalidJointChildWorld)
   auto errors = root.Load(testFile);
   for (auto e : errors)
     std::cout << e << std::endl;
-  ASSERT_EQ(7u, errors.size());
+  ASSERT_EQ(10u, errors.size());
   EXPECT_EQ(errors[0].Code(), sdf::ErrorCode::JOINT_CHILD_LINK_INVALID);
   EXPECT_NE(std::string::npos,
     errors[0].Message().find(
@@ -230,6 +230,70 @@ TEST(DOMJoint, LoadInvalidJointChildWorld)
       "not found in model with name[joint_child_world]"));
 }
 
+/////////////////////////////////////////////////
+TEST(DOMJoint, LoadJointParentModelFrame)
+{
+  const std::string testFile =
+    sdf::testing::TestFile("sdf", "joint_parent_model_frame.sdf");
+
+  // Load the SDF file
+  sdf::Root root;
+  sdf::Errors errors = root.Load(testFile);
+  EXPECT_TRUE(errors.empty()) << errors;
+
+  // Get the first model
+  const sdf::Model *model = root.Model();
+  ASSERT_NE(nullptr, model);
+  EXPECT_EQ("joint_parent_model_frame", model->Name());
+  EXPECT_EQ(2u, model->LinkCount());
+
+  EXPECT_EQ(1u, model->JointCount());
+  auto *joint = model->JointByName("joint");
+  ASSERT_NE(nullptr, joint);
+  EXPECT_EQ("child_link", model->JointByName("joint")->ChildLinkName());
+  EXPECT_EQ("__model__", model->JointByName("joint")->ParentLinkName());
+  std::string resolvedLinkName;
+  EXPECT_TRUE(joint->ResolveParentLink(resolvedLinkName).empty());
+  EXPECT_EQ("base_link", resolvedLinkName);
+
+  ignition::math::Pose3d pose;
+  EXPECT_TRUE(
+    joint->SemanticPose().Resolve(pose, "__model__").empty());
+  EXPECT_EQ(ignition::math::Pose3d(1, 0, 0, 0, 0, 0), pose);
+}
+
+/////////////////////////////////////////////////
+TEST(DOMJoint, LoadJointChildModelFrame)
+{
+  const std::string testFile =
+    sdf::testing::TestFile("sdf", "joint_child_model_frame.sdf");
+
+  // Load the SDF file
+  sdf::Root root;
+  sdf::Errors errors = root.Load(testFile);
+  EXPECT_TRUE(errors.empty()) << errors;
+
+  // Get the first model
+  const sdf::Model *model = root.Model();
+  ASSERT_NE(nullptr, model);
+  EXPECT_EQ("joint_child_model_frame", model->Name());
+  EXPECT_EQ(2u, model->LinkCount());
+
+  EXPECT_EQ(1u, model->JointCount());
+  auto *joint = model->JointByName("joint");
+  ASSERT_NE(nullptr, joint);
+  EXPECT_EQ("__model__", model->JointByName("joint")->ChildLinkName());
+  EXPECT_EQ("parent_link", model->JointByName("joint")->ParentLinkName());
+  std::string resolvedLinkName;
+  EXPECT_TRUE(joint->ResolveChildLink(resolvedLinkName).empty());
+  EXPECT_EQ("base_link", resolvedLinkName);
+
+  ignition::math::Pose3d pose;
+  EXPECT_TRUE(
+    joint->SemanticPose().Resolve(pose, "__model__").empty());
+  EXPECT_EQ(ignition::math::Pose3d(0, 0, 0, 0, 0, 0), pose);
+}
+
 /////////////////////////////////////////////////
 TEST(DOMJoint, LoadJointParentFrame)
 {
@@ -549,7 +613,7 @@ TEST(DOMJoint, LoadInvalidChild)
   for (auto e : errors)
     std::cout << e << std::endl;
   EXPECT_FALSE(errors.empty());
-  EXPECT_EQ(6u, errors.size());
+  EXPECT_EQ(8u, errors.size());
   EXPECT_EQ(errors[0].Code(), sdf::ErrorCode::JOINT_CHILD_LINK_INVALID);
   EXPECT_NE(std::string::npos,
     errors[0].Message().find(
@@ -564,6 +628,8 @@ TEST(DOMJoint, LoadInvalidChild)
   // errors[3]
   // errors[4]
   // errors[5]
+  // errors[6]
+  // errors[7]
 }
 
 /////////////////////////////////////////////////
@@ -584,7 +650,7 @@ TEST(DOMJoint, LoadLinkJointSameName17Invalid)
   for (auto e : errors)
     std::cout << e << std::endl;
   EXPECT_FALSE(errors.empty());
-  EXPECT_EQ(7u, errors.size());
+  EXPECT_EQ(9u, errors.size());
   EXPECT_EQ(errors[0].Code(), sdf::ErrorCode::DUPLICATE_NAME);
   EXPECT_NE(std::string::npos,
     errors[0].Message().find(
@@ -855,6 +921,38 @@ TEST(DOMJoint, LoadJointNestedParentChild)
     EXPECT_TRUE(j5->SemanticPose().Resolve(pose, "__model__").empty());
     EXPECT_EQ(Pose(0, -1, 1, IGN_PI_2, 0, 0), pose);
   }
+  {
+    const sdf::Joint *j6 = model->JointByName("J6");
+    ASSERT_NE(nullptr, j6);
+    EXPECT_EQ("M1::__model__", j6->ParentLinkName());
+    EXPECT_EQ("L1", j6->ChildLinkName());
+
+    std::string resolvedLinkName;
+    EXPECT_TRUE(j6->ResolveParentLink(resolvedLinkName).empty());
+    EXPECT_EQ("M1::L1", resolvedLinkName);
+    EXPECT_TRUE(j6->ResolveChildLink(resolvedLinkName).empty());
+    EXPECT_EQ("L1", resolvedLinkName);
+
+    Pose pose;
+    EXPECT_TRUE(j6->SemanticPose().Resolve(pose, "__model__").empty());
+    EXPECT_EQ(Pose(1, 0, 10, 0, IGN_PI_2, 0), pose);
+  }
+  {
+    const sdf::Joint *j7 = model->JointByName("J7");
+    ASSERT_NE(nullptr, j7);
+    EXPECT_EQ("L1", j7->ParentLinkName());
+    EXPECT_EQ("M1::__model__", j7->ChildLinkName());
+
+    std::string resolvedLinkName;
+    EXPECT_TRUE(j7->ResolveParentLink(resolvedLinkName).empty());
+    EXPECT_EQ("L1", resolvedLinkName);
+    EXPECT_TRUE(j7->ResolveChildLink(resolvedLinkName).empty());
+    EXPECT_EQ("M1::L1", resolvedLinkName);
+
+    Pose pose;
+    EXPECT_TRUE(j7->SemanticPose().Resolve(pose, "__model__").empty());
+    EXPECT_EQ(Pose(0, 0, 1, 0, 0, 0), pose);
+  }
 }
 
 
@@ -890,9 +988,24 @@ TEST(DOMJoint, Sensors)
   EXPECT_EQ(sdf::SensorType::FORCE_TORQUE, forceTorqueSensor->Type());
   EXPECT_EQ(ignition::math::Pose3d(10, 11, 12, 0, 0, 0),
       forceTorqueSensor->RawPose());
-  auto forceTorqueSensorConfig = forceTorqueSensor->ForceTorqueSensor();
-  ASSERT_NE(nullptr, forceTorqueSensorConfig);
-  EXPECT_EQ(sdf::ForceTorqueFrame::PARENT, forceTorqueSensorConfig->Frame());
+  auto forceTorqueSensorObj = forceTorqueSensor->ForceTorqueSensor();
+  ASSERT_NE(nullptr, forceTorqueSensorObj);
+  EXPECT_EQ(sdf::ForceTorqueFrame::PARENT, forceTorqueSensorObj->Frame());
   EXPECT_EQ(sdf::ForceTorqueMeasureDirection::PARENT_TO_CHILD,
-      forceTorqueSensorConfig->MeasureDirection());
+      forceTorqueSensorObj->MeasureDirection());
+
+  EXPECT_DOUBLE_EQ(0.0, forceTorqueSensorObj->ForceXNoise().Mean());
+  EXPECT_DOUBLE_EQ(0.1, forceTorqueSensorObj->ForceXNoise().StdDev());
+  EXPECT_DOUBLE_EQ(1.0, forceTorqueSensorObj->ForceYNoise().Mean());
+  EXPECT_DOUBLE_EQ(1.1, forceTorqueSensorObj->ForceYNoise().StdDev());
+  EXPECT_DOUBLE_EQ(2.0, forceTorqueSensorObj->ForceZNoise().Mean());
+  EXPECT_DOUBLE_EQ(2.1, forceTorqueSensorObj->ForceZNoise().StdDev());
+
+  EXPECT_DOUBLE_EQ(3.0, forceTorqueSensorObj->TorqueXNoise().Mean());
+  EXPECT_DOUBLE_EQ(3.1, forceTorqueSensorObj->TorqueXNoise().StdDev());
+  EXPECT_DOUBLE_EQ(4.0, forceTorqueSensorObj->TorqueYNoise().Mean());
+  EXPECT_DOUBLE_EQ(4.1, forceTorqueSensorObj->TorqueYNoise().StdDev());
+  EXPECT_DOUBLE_EQ(5.0, forceTorqueSensorObj->TorqueZNoise().Mean());
+  EXPECT_DOUBLE_EQ(5.1, forceTorqueSensorObj->TorqueZNoise().StdDev());
+
 }
