@@ -29,6 +29,7 @@
 #include "sdf/Types.hh"
 #include "sdf/World.hh"
 #include "test_config.h"
+#include "test_utils.hh"
 
 //////////////////////////////////////////////////
 TEST(DOMModel, NotAModel)
@@ -308,6 +309,23 @@ TEST(DOMRoot, MultiNestedModel)
   EXPECT_EQ(innerModel->FrameByIndex(0),
             outerModel->FrameByName(innerFrameNestedName));
   EXPECT_NE(nullptr, outerModel->FrameByName(innerFrameNestedName));
+
+
+  // Check that each implicit/explicit frame is in the frame attached to graph
+  EXPECT_TRUE(outerModel->NameExistsInFrameAttachedToGraph("outer_link"));
+  EXPECT_TRUE(outerModel->NameExistsInFrameAttachedToGraph("outer_joint"));
+  EXPECT_TRUE(outerModel->NameExistsInFrameAttachedToGraph("outer_frame"));
+  EXPECT_TRUE(outerModel->NameExistsInFrameAttachedToGraph("mid_model"));
+
+  // Check that mid_link does not exist directly under outer_model, but can be
+  // accessed via its scoped name
+  EXPECT_FALSE(outerModel->NameExistsInFrameAttachedToGraph("mid_link"));
+  EXPECT_TRUE(
+      outerModel->NameExistsInFrameAttachedToGraph("mid_model::mid_link"));
+
+  // Check multiple levels of nesting
+  EXPECT_TRUE(outerModel->NameExistsInFrameAttachedToGraph(
+      "mid_model::inner_model::inner_joint"));
 }
 
 /////////////////////////////////////////////////
@@ -486,6 +504,63 @@ TEST(DOMRoot, LoadNestedCanonicalLink)
   EXPECT_EQ("deep::deeper::deepest::deepest_link", body);
 }
 
+/////////////////////////////////////////////////
+TEST(DOMModel, LoadModelWithDuplicateChildNames)
+{
+  // Redirect sdfwarn output
+  std::stringstream buffer;
+  sdf::testing::RedirectConsoleStream redir(
+      sdf::Console::Instance()->GetMsgStream(), &buffer);
+
+#ifdef _WIN32
+  sdf::Console::Instance()->SetQuiet(false);
+  sdf::testing::ScopeExit revertSetQuiet(
+      []
+      {
+        sdf::Console::Instance()->SetQuiet(true);
+      });
+#endif
+
+  {
+    buffer.str("");
+    const std::string testFile =
+      sdf::testing::TestFile("sdf", "model_link_joint_same_name.sdf");
+
+    // Load the SDF file
+    sdf::Root root;
+    auto errors = root.Load(testFile);
+    EXPECT_TRUE(errors.empty());
+    for (const auto &err : errors)
+    {
+      std::cout << err << std::endl;
+    }
+
+    // Check warning message
+    EXPECT_NE(
+        std::string::npos,
+        buffer.str().find("Non-unique name[attachment] detected 2 times in XML "
+                          "children of model with name[link_joint_same_name]"))
+        << buffer.str();
+  }
+
+  // Check that there's an exception for "plugin" elements
+  {
+    buffer.str("");
+    const std::string testFile =
+      sdf::testing::TestFile("sdf", "model_duplicate_plugins.sdf");
+
+    // Load the SDF file
+    sdf::Root root;
+    auto errors = root.Load(testFile);
+    EXPECT_TRUE(errors.empty());
+    for (const auto &err : errors)
+    {
+      std::cout << err << std::endl;
+    }
+    EXPECT_TRUE(buffer.str().empty()) << buffer.str();
+  }
+}
+
 /////////////////////////////////////////////////
 TEST(DOMRoot, LoadNestedExplicitCanonicalLink)
 {
